/*---------------------------------------------------------------------------
 *
 *      Title:          Dynamic Memory Allocation tracking and checking
 *
 *      Purpose:        Debugging tool
 *
 *      File:           dyn_val.h
 *
 *      Creation Date:  August 1992
 *
 *      Description:
 *             Provide replacement for standard allocation routines which track
 *      all allocations, provide error notification if freeing non-allocated memory
 *      allow dumping of what memory is in use (good for checking for leaks), and
 *      automatically verify for overwrites past start & end.
 *      It also fills all free'd & delete'd blocks with 0xfe to prevent possible
 *      usage after free
 *
 *      Data sources:
 *
 *      1.0     R.J.Harrison    Implement
 *      1.1     R.J.Harrison    Modifications for speed          February 1993
 *      1.2     R.J.Harrison    Complete re-write                November 1994
 *      1.3     R.J.Harrison    Generic PC version               August   1995
 *      1.4     R.J.Harrison    Shareware release changes        June     1996
 *
 *      Copyright  1991-1996 R.J.Harrison
 *---------------------------------------------------------------------------*/
#ifndef DV_ALLOC_TRACKING
#define DV_ALLOC_TRACKING

#if defined(DEBUG) || defined(_DEBUG) || defined(DV_DYN_VAL)
#ifdef  __cplusplus
extern "C" {
#endif
#define SAVESTR_CHECK

#define TY_int long 

#ifdef calloc
#undef calloc
#endif
#ifdef free
#undef free
#endif
#ifdef malloc
#undef malloc
#endif
#ifdef savestr
#undef savestr
#endif

#ifndef MODULE
#define MODULE __FILE__
#endif

#define calloc(a,b) DV_calloc(a,b,MODULE,__LINE__)
#define realloc(a,b) DV_realloc(a,b,MODULE,__LINE__)
#define malloc(a) DV_malloc(a,MODULE,__LINE__)
#define free(a) DV_free(a,MODULE,__LINE__)
#define savestr(a) DV_savestr(a,MODULE,__LINE__)
#define strdup(a) DV_savestr(a,MODULE,__LINE__)
#define tempnam(a,b) DV_tempnam(a,b,MODULE,__LINE__)
#define tmpnam(a) DV_tmpnam(a,MODULE,__LINE__)

#define FW_calloc(a,b,c) _FW_calloc(a,b,c,MODULE,__LINE__)
#define FW_malloc(a,b) _FW_malloc(a,b,MODULE,__LINE__)
#define FW_free(a,b) _FW_free(a,b,MODULE,__LINE__)
#define FW_savestr(a,b) _FW_savestr(a,b,MODULE,__LINE__)

/*
 * Prototypes (user functions)
 */
/*
 * Name:       DV_ov_check_memory
 * Purpose:    Check memory for overwrites
 * Parameters: None
 */
void DV_ov_check_memory (void);

/*
 * Name:       DV_get_dynamic_memory_usage
 * Purpose:    Return the amount (in bytes) of dynamic memory in use
 * Parameters: None
 */
TY_int DV_get_dynamic_memory_usage (void);

/*
 * Name:       DV_scan_check_memory 
 * Purpose:    Check memory for overwrites, call as often as required
 *             (e.g. in an event loop) as it will only check every second
 * Parameters: None
 */
void DV_scan_check_memory (void);

/*
 * Name:       DV_dump_memory 
 * Purpose:    Output to a file, all memory currently allocated
 *             Writes to file memory##.usage, where ## starts at 1
 *             and increments every time called
 *             Useful for spotting leaks.
 * Parameters: None
 */
void DV_dump_memory (void);


/*
 * Prototypes (user functions) - don't call these unless you're really
 * sure, etc...
 */
void *DV_get (unsigned long sz);
void DV_track (unsigned long sz, void *ptr, char *module, long line);
void *DV_calloc (unsigned TY_int ne,   unsigned TY_int sz,   char *module,   TY_int line);
void *DV_malloc (unsigned TY_int sz,   char *module,   TY_int line);
void *DV_realloc (void *blk, unsigned long new_sz, char *module, long line);
void DV_free (void *a,       char *module,       TY_int line);
void *DV_savestr (char *p, char *module, TY_int line);
char *DV_tempnam(char *, char *, char*, TY_int);
char *DV_tmpnam(char *);

#ifdef  __cplusplus
}
void *DV_new_set_id(char *fn, int line=0);
#define NEW_MARK DV_new_set_id(__FILE__, __LINE__);
#define NEW_ID(a) DV_new_set_id(a);
#endif

#else
# ifdef  __cplusplus
# define DV_new_set_id(a,b)
#  define NEW_MARK 
#  define NEW_ID(a)
# endif
#endif
#endif

